package org.seasar.util.text.env.impl;

import java.io.InputStream;
import java.net.URL;

import ognl.OgnlRuntime;

import org.apache.commons.configuration.ConfigurationUtils;
import org.seasar.util.text.env.Environment;
import org.seasar.util.text.env.KeyNotFoundException;
import org.seasar.util.text.env.LoadingFailedException;
import org.seasar.util.text.env.ognl.EnvironmentPropertyAccessor;

/**
 * ϐ̂悤ɎQƂ邱Ƃ̏oAL[ƒl̃}bsOێNX̒ێ񋟂܂B
 * 
 * @author Koichi Kobayashi
 */
public abstract class AbstractEnvironment implements Environment {
    protected boolean throwKeyNotFoundException = true;
    protected String basePath;

    static {
        OgnlRuntime.setPropertyAccessor(Environment.class, new EnvironmentPropertyAccessor());
    }

    /**
     * <code>key</code> ɑΉtꂽlԂ܂B <br>
     * <code>throwKeyNotFoundException</code>  <code>true</code> łA
     * <code>key</code> ɒlΉtĂȂꍇ́A <code>KeyNotFoundException</code>
     * X[܂B <br>
     * <code>throwKeyNotFoundException</code>  <code>true</code> łȂA
     * <code>key</code> ɒlΉtĂȂꍇ́A <code>null</code> Ԃ܂B
     * 
     * @param key
     *            L[
     * @return <code>key</code> ɑΉtꂽl
     * @exception <code>key</code> ɒlΉtĂȂꍇɃX[܂ (
     *                <code>throwKeyNotFoundException</code> 
     *                <code>true</code> ɐݒ肳Ăꍇ̂)
     */
    public String get(final String key) {
        final String value = getProperty(key);
        if (throwKeyNotFoundException && value == null) {
            throw new KeyNotFoundException(key);
        }
        return value;
    }

    /**
     * vpeB throwKeyNotFoundException ̒lԂ܂B
     * 
     * @return Returns the raiseExceptionIfNull.
     */
    public boolean isThrowKeyNotFoundException() {
        return throwKeyNotFoundException;
    }

    /**
     * vpeB setThrowKeyNotFoundException ̒lݒ肵܂B
     * 
     * @param raiseExceptionIfNull
     *            The raiseExceptionIfNull to set.
     */
    public void setThrowKeyNotFoundException(final boolean raiseExceptionIfNull) {
        this.throwKeyNotFoundException = raiseExceptionIfNull;
    }

    /**
     * vpeB basePath ̒lԂ܂B
     * 
     * @return Returns the basePath.
     */
    public String getBasePath() {
        return basePath;
    }

    /**
     * vpeB basePath ̒lݒ肵܂B
     * 
     * @param basePath
     *            The basePath to set.
     */
    public void setBasePath(String basePath) {
        this.basePath = basePath;
    }

    /**
     * <code>resourceName</code> Ŏꂽ\[Xǂݍ <code>InputStream</code>
     * Ԃ܂B
     * 
     * @param resourceName
     *            \[X
     * @return \[Xǂݍ <code>InputStream</code>
     * @exception LoadingFailedException
     *                \[X̓ǂݍݒɃG[ꍇɃX[܂
     */
    protected InputStream getResourceAsInputStream(final String resourceName) {
        try {
            final URL url = ConfigurationUtils.locate(basePath, resourceName);
            if (url == null) {
                throw new LoadingFailedException();
            }
            return url.openStream();
        }
        catch (final Exception e) {
            throw new LoadingFailedException(e);
        }
    }

    /**
     * <code>key</code> ɑΉtꂽlԂ܂B <br>
     * <code>key</code> ɒlΉtĂȂꍇ́A <code>null</code> Ԃ܂B
     * 
     * @param key
     *            L[
     * @return <code>key</code> ɑΉtꂽl
     */
    protected abstract String getProperty(String key);
}