package org.seasar.util.text.env.impl;

import java.util.Properties;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.seasar.framework.util.SAXParserFactoryUtil;
import org.seasar.framework.xml.SaxHandler;
import org.seasar.framework.xml.SaxHandlerParser;
import org.seasar.framework.xml.TagHandler;
import org.seasar.framework.xml.TagHandlerContext;
import org.seasar.framework.xml.TagHandlerRule;
import org.xml.sax.Attributes;

/**
 * ϐ̂悤ɎQƂ邱Ƃ̏oAL[ƒl̃}bsOێNXłB <br>
 * ̎́C}bsO J2SE5.0 œꂽ XML vpeBhLgɗގ XML t@C烍[h܂B XML
 * t@Cɂ͎̕^錾KvłB
 * 
 * <pre>
 * 
 *  
 *   
 *     &lt;!DOCTYPE properties PUBLIC &quot;-//SEASAR//DTD S2Util-Text-Env//EN&quot; &quot;http://www.seasar.org/dtd/s2util-text-env.dtd&quot;&gt;
 *    
 *   
 *  
 * </pre>
 * 
 * @author Koichi Kobayashi
 */
public class XMLEnvironment extends AbstractEnvironment {
    public static final String PUBLIC_ID = "-//SEASAR//DTD S2Util-Text-Env//EN";
    public static final String DTD_PATH = "org/seasar/util/text/env/s2util-text-env.dtd";

    protected final Properties properties = new Properties();

    /**
     * <code>key</code> ɑΉtꂽlԂ܂B <br>
     * <code>key</code> ɒlΉtĂȂꍇ́A <code>null</code> Ԃ܂B
     * 
     * @param key
     *            L[
     * @return <code>key</code> ɑΉtꂽl
     */
    protected String getProperty(final String key) {
        return properties.getProperty(key);
    }

    /**
     * \[Xǂݍ݂܂B
     * 
     * @param resourceName
     *            \[X
     */
    public void load(final String resourceName) {
        final SAXParserFactory factory = SAXParserFactoryUtil.newInstance();
        factory.setValidating(true);

        final SAXParser saxParser = SAXParserFactoryUtil.newSAXParser(factory);

        final SaxHandler handler = new SaxHandler(new XMLEnvironmentRule());
        handler.registerDtdPath(PUBLIC_ID, DTD_PATH);

        final TagHandlerContext ctx = handler.getTagHandlerContext();
        ctx.addParameter("properties", properties);

        final SaxHandlerParser parser = new SaxHandlerParser(handler, saxParser);
        parser.parse(getResourceAsInputStream(resourceName));
    }

    static class XMLEnvironmentRule extends TagHandlerRule {
        public XMLEnvironmentRule() {
            addTagHandler("entry", new EntryTagHandler());
        }
    }

    static public class EntryTagHandler extends TagHandler {
        public void start(final TagHandlerContext context, final Attributes attributes) {
            context.push(attributes.getValue("key"));
        }

        public void end(final TagHandlerContext context, final String body) {
            if (body != null) {
                final String value = body.trim();
                if (value.length() > 0) {
                    final String key = (String) context.pop();
                    final Properties properties = (Properties) context.getParameter("properties");
                    properties.setProperty(key, body.trim());
                }
            }
        }
    }
}